var sensorData = require('../../libs/sensorData'),
    actionData = require('../../libs/actionData')
    _ = require('underscore');

/**
 * Return a collection of sensorData objects for all types of sensors
 * present in the provided collection
 */
module.exports.sensorTypes = function (sensors) {
  var types = {};
  
  sensors.forEach(function(sensor) {
    var typeId = sensor.get('type');
    var sensorType = sensorData[typeId];

    if (!sensorType)
      return;

    if (!types[typeId])
      types[typeId] = sensorType;
  });

  var sorted = [];
  for (var t in types) { sorted.push({type: t, data: types[t]}); }
  
  sorted = sorted.sort(function(a, b) { return a.data.name.localeCompare(b.data.name); });

  return sorted;
}

/**
 * Return a collection of sensorData objects for all types of sensors
 * present in the provided collection, excluding any types which do
 * not have actions
 */
module.exports.sensorTypesWithActions = function (sensors) {
  return module.exports.sensorTypes(sensors)
    .filter(function(type) { return type.data.actions && type.data.actions.length > 0; });
}

/**
 * Return the placement string for the popover placement
 */
module.exports.popoverPlacement = function (idx) {
  var col = idx % 4;
  
  if (col == 0)
    return 'right';
  /*
  else if (col == 4)
    return 'left';
  */
  else
    return 'bottom';
  
}

/**
 * Filter a set of sensors by type
 */
module.exports.sensorsForType = function (sensors, type) {
  return sensors.filter(function(s) { return s.get('type') == type; });
}

/**
 * Filter a set of sensors by multiple types
 */
module.exports.sensorsForTypes = function (sensors, types) {
  if ((!types) || (types.length == 0))
    return sensors;
  
  return sensors.filter(function(s) { return types.indexOf(s.get('type')) !== -1; });
}

/**
 * Humanize a given string
 */
module.exports.humanize = function(str) {
  str = str.replace(/([A-Z]+)([A-Z][a-z])/, "$1 $2");
  str = str.replace(/([a-z\d])([A-Z])/, "$1 $2");
  str = str.replace(/[-_]+/, ' ');

  return str.replace(/^([a-z])/, function(match, p1) { return p1.toUpperCase(); });
}

/**
 * Utility returns true if any value from collectionA also exists in collectionB
 */
function anyMutual(collectionA, collectionB) {
  return _.find(collectionA, function(item) {
    return collectionB.indexOf(item) !== -1;
  });
}

/**
 * Return all allowed actions for a given set of sensors
 */
module.exports.allowedRuleActions = function(sensors) {
  var types = sensors.map(function(s) { return s.get('type'); });
  
  return actionData.actions.filter(function(action) {

    var invalidSensorArg = _.find(action.args, function(arg) {
      if (arg.type != 'sensor_id' && arg.type != 'sensor_mac')
        return false;
      
      return ( arg.allowableTypes && (!anyMutual(arg.allowableTypes, types)) );
    });
    
    return (!invalidSensorArg);
    
  }).sort(function(a, b) {
    
    return a.name.localeCompare(b.name);
    
  });
}

/**
 * Return true if the action has any arguments requiring user input
 */
module.exports.actionHasUserArg = function(action) {
  return _.find(action.args, function(arg) {
    return arg.type == 'sensor_id' || arg.type == 'sensor_mac' || arg.type == 'string' || arg.type == 'string';
  });
}

/**
 * Return the data-dipslay-requirements value for a given action's button
 */
module.exports.displayRequirementsForAction = function(action) {

  var events = '';
  if (action.allowedEvents)
    events = action.allowedEvents.join(',');

  var subs = '';
  if (action.allowedSensorTypes)
    subs = action.allowedSensorTypes.join(',');
  
  if (action.allowedProviderTypes)
    subs += subs == '' ? action.allowedProviderTypes.join(',') : (','+action.allowedProviderTypes.join(','));

  return events + '|' + subs;
}
